.\" -*-nroff-*-
.\" =========> WARNING: tab characters are needed in this file <========
.\" ====================================================================
.\"  @Troff-man-file{
.\"     author          = "Nelson H. F. Beebe",
.\"     version         = "3.0.0",
.\"     date            = "21 May 2001",
.\"     time            = "05:40:30 MDT",
.\"     filename        = "fpp.man",
.\"     copyright       = "Copyright (c) 1990, 1993, 2001 Nelson H. F.
.\"                        Beebe. This code is licensed under the GNU
.\"                        General Public License, version 2 or
.\"                        later.",
.\"     address         = "Center for Scientific Computing
.\"                        University of Utah
.\"                        Department of Mathematics, 322 INSCC
.\"                        155 S 1400 E RM 233
.\"                        Salt Lake City, UT 84112-0090
.\"                        USA",
.\"     telephone       = "+1 801 581 5254",
.\"     FAX             = "+1 801 585 1640, +1 801 581 4148",
.\"     URL             = "http://www.math.utah.edu/~beebe",
.\"     checksum        = "52445 1441 5147 37117",
.\"     email           = "beebe@math.utah.edu, beebe@acm.org,
.\"                        beebe@computer.org, beebe@ieee.org
.\"                        (Internet)",
.\"     codetable       = "ISO/ASCII",
.\"     keywords        = "Fortran;  PLOT79; preprocessor; SFTRAN3",
.\"     supported       = "yes",
.\"     docstring       = "This file provides the UNIX manual pages that
.\"                        document fpp, a cpp-like reversible
.\"                        preprocessor filter for Fortran and SFTRAN3
.\"                        code.
.\"
.\"                        The checksum field above contains a CRC-16
.\"                        checksum as the first value, followed by the
.\"                        equivalent of the standard UNIX wc (word
.\"                        count) utility output of lines, words, and
.\"                        characters.  This is produced by Robert
.\"                        Solovay's checksum utility.",
.\"  }
.\" ====================================================================
.TH FPP 1L "17 May 2001" "" "Version 3.0.0"
.\"======================================================================
.SH "NAME"
fpp \- cpp-like reversible preprocessor filter for Fortran and SFTRAN3 code
.\"======================================================================
.SH "SYNOPSIS"
.B fpp
[
.B \-?
]
[
.B \-author
]
[
.B \-copyright
]
[
.BI \-D name
]
.if n .ti +\w'\fBfpp\fP\ 'u
[
.BI \-D name = value
]
[
.B \-debug
]
[
.B \-fixed
]
[
.B \-free
]
.if n .ti +\w'\fBfpp\fP\ 'u
.if t .ti +\w'\fBfpp\fP\ 'u
[
.B \-help
]
[
.BI \-U name
]
[
.B \-version
]
[
.B name1=value1
]
\&.\|.\|.
.if n .ti +\w'\fBfpp\fP\ 'u
[
.B namek=valuek
]
[
.B \-\^\-
]
.if t .ti +\w'\fBfpp\fP\ 'u
[
.I infile(s)
]
.\"======================================================================
.SH "DESCRIPTION"
.PP
.B fpp
is a preprocessor for Fortran (77, 90, 95) and SFTRAN3, modelled on
the ISO Standard C (1989 and 1999) and C++ (1998) preprocessor, but
tailored for Fortran use, and for
.IR reversibility .
The output always contains the complete input, except that some code
sections may have become comments, or vice versa.  This can be more
useful than the non-reversible approach used by the C preprocessor.
.PP
Reversible preprocessing is convenient when a master source file
must be maintained to generate multiple versions, such as for
different operating-system, compiler, and architecture
variations.  The source code for any of these can serve as the
master file to create any of the others.
.\" .PP
.\" The current implementation is written in
.\" .BR nawk (1).
.\" That language is available on several different operating
.\" systems, in both commercial and free implementations, and serves
.\" as a convenient prototyping language for a program such as
.\" .BR fpp .
.PP
Although several UNIX vendor Fortran compilers offer to run
the C preprocessor,
.BR cpp (1),
it cannot be used reliably for Fortran code, because it does not know
about Fortran line-length limitations, or Fortran syntax.  Fortran
does not require type declarations, so from syntax confusion,
.BR cpp (1)
could introduce subtle changes that cannot be detected by the Fortran
compiler, and would likely be quite hard to debug.
.PP
A similar problem exists with the more general and powerful macro
processor,
.BR m4 (1).
.PP
.B fpp
follows the preprocessor syntax defined in three ISO language
standards, but embeds directives in comments and restricts its
operation, so that both input and output files can be processed by any
Fortran compiler, or any other Fortran software tool, such as
.BR d2s (1),
.BR dtoq (1),
.BR dtos (1),
.BR fsplit (1),
.BR ftnchek (1),
.BR pfort (1),
.BR pretty (1),
.BR qtod (1),
.BR s2d (1),
.BR sf3lex (1),
.BR sf3pretty (1),
.BR stod (1),
.BR strsf3 (1),
.BR struct (1),
.BR tidy (1),
or
.BR toolpack (1).
Also, a human can see exactly the same post-processed source code that
the compiler does.
.PP
In particular, this means that it is often possible to apply
.B fpp
just
.I once
to a set of Fortran source files for a given environment, rather than
having to run it each time the code is compiled.
.\"======================================================================
.SH "OPTIONS"
Because the original version of
.B fpp
was implemented in the
.BR awk (1)
language, which has its own command-line options,
.B fpp
options were originally prefixed with a
.B +
instead of a
.BR \- .
That syntax is still recognized, as is the GNU/POSIX form with a
leading double hyphen.  Also, long option names may be abbreviated
to any unique leading prefix.
Thus,
.BR +author ,
.BR \-author ,
.BR \-\^\-author ,
and
.B \-a
are equivalent.
.\"---------------------------------------------------------------------
.TP \w'\fB\-D\fP\fIname=value\fP'u+2n
.B \-\^\-
The preceding word on the command line is the last option; all
following words are to be interpreted as filenames, even if they
begin with a hyphen, or contain equal signs.
.\"---------------------------------------------------------------------
.TP
.B \-?
Display brief usage information on
.I stderr
and exit with a success status code before
processing any input files.
.IP
This is a synonym for
.BR \-help .
.\"---------------------------------------------------------------------
.TP
.B \-author
Show author information on
.I stderr
and exit with a success status code before processing any input files.
.\"---------------------------------------------------------------------
.TP
.B \-copyright
Show copyright information on
.I stderr
and exit with a success status code before processing any input files.
.\"---------------------------------------------------------------------
.TP
.BI \-D name
Define the symbol
.I name
to the value 1.
.\"---------------------------------------------------------------------
.TP
.BI \-D name=value
Define the symbol
.I name
to
.IR value .
.\"---------------------------------------------------------------------
.TP
.B \-debug
Turn on debugging output, which is sent to
.IR stderr .
This produces helpful intermediate output from the expression
evaluator.  Macro definitions are also displayed on
.I stderr
when they are executed.
.\"---------------------------------------------------------------------
.TP
.BR \-fixed
Input is in Fortran 77 fixed form.  If this option is specified, it
overrides any assumptions based on input file extensions.
.IP
Fixed-form input is the default.
.IP
In the absence of a command-line option to set the input form,
fixed form is automatically selected for files with names ending in
.IR .F ,
.IR .f ,
.IR .F77 ,
.IR .f77 ,
.IR .FOR ,
.IR .for ,
.IR .FPP ,
.IR .fpp ,
.IR .FTN ,
or
.IR .ftn ,
or any unrecognized extension.
.\"---------------------------------------------------------------------
.TP
.BR \-free
Input is in Fortran 90 and 95 free form.  If this option is specified,
it overrides any assumptions based on input file extensions.
.IP
In the absence of a command-line option to set the input form,
free form is automatically selected for files with names ending in
.IR .F90 ,
.IR .f90 ,
.IR .F95 ,
or
.IR .f95 .
.\"---------------------------------------------------------------------
.TP
.B \-help
Display brief usage information on
.I stderr
and exit with a success status code before processing any input files.
.IP
This is a synonym for
.BR \-? .
.\"---------------------------------------------------------------------
.TP
.BI \-U name
Undefine the symbol
.IR name .
If the name is subsequently referenced, it will
.I silently
evaluate to zero.  The existence of a definition can be checked with
the
.B defined
operator, or in
.IP
.RS
.nf
.\" NB: We use bold fonts here, so we need digit-width spaces, \0
.BR "C#ifdef"  "\0\0\0name"
.BR "C#ifndef" "\0\0\0name"
.fi
.RE
.IP
statements;  see below for details.
.\"---------------------------------------------------------------------
.TP
.B \-version
Display the current version number of
.B fpp
on
.IR stderr ,
and exit with a success status code before processing any input files.
.\"---------------------------------------------------------------------
.TP
.B name=value
Define the symbol
.I name
to
.IR value .
This form is deprecated, and may not be supported in the future.  Use
.BI \-D name=value
instead.
.\"---------------------------------------------------------------------
.PP
If no input file names are given on the command line, input
is assumed to come from
.IR stdin .
In that case, since no filename is available to select the input form,
it will be necessary to specify
.B \-free
for free-form source code.
.\"======================================================================
.SH "LANGUAGE OVERVIEW"
.B fpp
statements, or directives, are specially-formatted Fortran
comments of the forms
.PP
.RS
.nf
.B "C#name"
.BR "C#name" "  args"
.fi
.RE
.PP
Since all
.B fpp
directives are encoded as comments, both input and output files
should be compilable without any preprocessing by
.BR fpp .
.PP
Blanks may optionally surround the initial
.B #
to permit indentation for better visibility, or to reflect
conditional statement nesting.
.PP
The first column may contain any valid Fortran comment starter:
.BR C ,
.BR c ,
.BR * ,
or
.BR ! .
.PP
Unrecognized
.B "C # word"
sequences are
.I silently
copied to the output, so as to permit the
rare case of a
.B #
in the initial text of a Fortran comment.
.PP
Preprocessor names in conditionals and definitions, or set on the
command line, consist of letters, digits, and underscores; the
first character may not be a digit.
.PP
Symbols beginning with two underscores, or an underscore and an
uppercase letter, are reserved for the local implementation; see
the
.I "PREDEFINED SYMBOLS"
section below for details.
.PP
Symbols beginning with two underscores are permanent: once defined,
they can be neither undefined, nor redefined, by the user.
.PP
Following standard Fortran practice, letter case is not
significant in directives, or in constants and operators in
expressions.
.\" However, if
.\" .BR nawk (1)
.\" is used to implement
.\" .BR fpp ,
.\" letter case is significant in names that are defined.  With
.\" .BR gawk (1),
.\" this limitation is removed; letter case is never significant.
.PP
For portability, it is recommended that lower-case letters be
used for all directives, and upper-case letters for all defined
names; this conforms to three decades of widespread practice in the
C programming language.
.\"======================================================================
.SH "DEFINITION STATEMENTS"
Definitions of names for preprocessor conditionals may be set on
the command line:
.PP
.RS
.nf
.B "fpp\0\0\0\-D_OS_UNIX\0\0\0\-D_SUN386"
.B "fpp\0\0\0\-D_OS_UNIX=1\0\0\0\-D_SUN386=1"
.B "fpp\0\0\0_OS_UNIX=1\0\0\0_SUN386=1"
.fi
.RE
.PP
or in the input file text itself:
.PP
.RS
.nf
.B "C\0#define\0\0\0_OS_UNIX\0\0\01"
.B "C\0#define\0\0\0_SUN386\0\0\01"
.fi
.RE
.PP
The
.BR cc (1)-like
forms with define and undefine options are supported;
these two invocations are equivalent:
.PP
.RS
.nf
.B "fpp\0\0\0-D_OS_UNIX\0\0\0-DWORDSIZE=32"
.B "fpp\0\0\0_OS_UNIX=1\0\0\0WORDSIZE=32"
.fi
.RE
.PP
These two are roughly equivalent:
.PP
.RS
.nf
.B "fpp\0\0\0-U_OS_VAXVMS"
.B "fpp\0\0\0_OS_VAXVMS=0"
.fi
.RE
.PP
They differ in that, although the symbol
.B _OS_VAXVMS
will evaluate to 0 in a numeric context, a test for definition with
.RS
.B "C #if defined(_OS_VAXVMS)"
.RE
or
.RS
.B "C #ifdef _OS_VAXVMS"
.RE
will select the else-branch in the first case, and the then-branch in
the second case.
.PP
If the value is omitted, as in
.PP
.RS
.nf
.B "fpp\0\0\0_OS_UNIX=\0\0\0_SUN386="
.fi
.RE
.PP
or
.PP
.RS
.nf
.B "C\0#define\0\0\0_OS_UNIX"
.B "C\0#define\0\0\0_SUN386"
.fi
.RE
.PP
a value of 1 is assumed.
.PP
Names can be undefined by
.PP
.RS
.nf
.BR "C\0#undef"  "\0\0name"
.BR "C\0#undefine" "\0\0name"
.fi
.RE
.PP
If the name was not already defined, the request is
.I silently
ignored.
.\"======================================================================
.SH "CONDITIONAL STATEMENTS"
The conditional statements supported are:
.PP
.RS
.nf
.BR "C#if"     "\0\0\0constant-expression"
.BR "C#ifdef"  "\0\0\0name"
.BR "C#ifndef" "\0\0\0name"
.BR "C#elseif" "\0\0\0constant-expression"
.BR "C#elif"   "\0\0\0constant-expression"
.BR "C#else"   "\0\0\0[optional comment]"
.BR "C#endif"  "\0\0\0[optional comment]"
.fi
.RE
.PP
Each
.B C#ifxxx
statement must have a matching
.B C#endif
following it.  The two may be separated by any number
of
.B C#elseif
statements, which may be followed by a single
.B C#else
statement.
.PP
A branch of a conditional is selected when the
expression evaluates to a non-zero value; see the
.I "EXPRESSIONS"
section below for details.
.PP
Code between these statements is preserved, but in the
unselected branches of a conditional statement, a
non-comment statement will be altered to a comment by
prefixing it with an initial
.BR C## ,
(or
.B !##
in free form) shifting the statement right by three columns.  In the
selected branch, any initial
.B C##
in columns 1 through 3 is stripped; lines without this prefix are
copied verbatim.
.PP
Because of Fortran line-length limitations (72 in fixed form, 132 in
free form), this means that inside an
.B fpp
conditional, code lines may not exceed three characters less than the
maximum length.
.PP
For example, the input
.PP
.RS
.nf
.ta (\w'MMMMMM'u)
\&\fBC#if\0_OS_UNIX
C##C\0\0\0\0\0UNIX\0code
C##\0\0\0\0\0\0CALL\0GETENV(...)
C#elseif\0_OS_VAXVMS
C\0\0\0\0\0VMS\0code
\0\0\0\0\0\0	CALL\0LIB$TRNLNM(...)
C#endif\fP
.fi
.RE
.PP
when
.B _OS_UNIX=1
produces
.PP
.RS
.nf
\&\fBC#if\0_OS_UNIX
C\0\0\0\0\0	UNIX\0code
\0\0\0\0\0\0	CALL\0GETENV(...)
C#elseif\0_OS_VAXVMS
C##C\0\0\0\0\0VMS\0code
C##\0\0\0\0\0\0CALL\0LIB$TRNLNM(...)
C#endif\fP
.fi
.RE
.PP
When neither
.B _OS_UNIX
nor
.B _OS_VAXVMS
are defined, the output is
.PP
.RS
.nf
\&\fBC#if\0_OS_UNIX
C##C\0\0\0\0\0UNIX\0code
C##\0\0\0\0\0\0	CALL\0GETENV(...)
C#elseif\0_OS_VAXVMS
C##C\0\0\0\0\0VMS\0code
C##\0\0\0\0\0\0CALL\0LIB$TRNLNM(...)
C#endif\fP
.fi
.RE
.PP
When only
.B _OS_VAXVMS
is defined, the original input is sent to the output.
If by chance both
.B _OS_VAXVMS
and
.B _OS_UNIX
were defined, only the UNIX code would be selected,
because only the first branch of the conditional would
be executed.
.PP
Preprocessor conditionals may be nested:
.PP
.RS
.nf
.B "C\0#\0if\0_OS_UNIX"
.B "C\0\0\0#\0if\0_SUN"
.B "C\0\0\0\0\0\0#\0if\0_SUN4"
.B "C\0\0\0\0\0\0#\0elseif\0_SUN3"
.B "C\0\0\0\0\0\0#\0elseif\0_SUN386"
.B "C\0\0\0\0\0\0#\0endif"
.B "C\0\0\0#\0endif"
.B "C\0#\0elseif\0_OS_VMS"
.B "C\0#\0endif"
.fi
.RE
.PP
Any text following
.B #else
or
.B #endif
is ignored; it can be used to document the conditional, usually
with the test from the preceding
.BR #if :
.PP
.RS
.nf
.B "C\0#\0if\0_OS_UNIX"
.B "C\0#\0else\0NOT\0_OS_UNIX"
.B "C\0#\0endif\0_OS_UNIX"
.fi
.RE
While such trailing text is not permitted by the ISO Standard C/C++
preprocessor, many implementations of the preprocessor silently ignore
such text.
.PP
.B fpp
directives are not executed if they are in a branch of a
conditional that is not currently selected.  However, conditional
statements are processed to keep track of the current nesting.
.PP
On UNIX, the
.B \-Dname
option for
.BR diff (1)
can be used to get output from the comparison of two
files that is almost correct input for
.BR fpp .
A simple command pipe
.B "diff -Dxxx file1 file2 | sed -e 's/^#/C#/' >file3
will produce an output file
.I file3
from which
.B "fpp -Dxxx file3"
will recover
.I file2
and
.B "fpp -Uxxx file3"
will recover
.IR file1 .
.\"======================================================================
.SH "EXPRESSIONS"
Expressions are recognized and evaluated in two circumstances: in
the arguments of
.BR C#if ,
.BR C#elseif ,
and
.BR C#elif ,
and inside the parentheses of
.BR #(.\|.\|.) .
.PP
In expressions, primaries are Fortran integer, floating-point,
logical, and character constants, and preprocessor names.
.PP
Undefined names
.I silently
evaluate to zero in arithmetic
expressions, and to empty strings in string expressions.
.PP
Character strings appearing in arithmetic expressions are
converted to numbers, which are zero if the string does not look
like a number.  Character strings appearing by themselves
evaluate to themselves.
.PP
Arithmetic expressions are evaluated in
.I floating-point
arithmetic; for Boolean (Fortran logical) tests, zero is false,
and non-zero is true.
.PP
The usual Fortran arithmetic operators
.B +\0\-\0*\0/\0**
are recognized, along with the C modulus operator
.BR % ;
.BR "x % y" " is Fortran's " "mod(x,y)" .
This operator is rigorously defined for all arguments
to be
.BR "x % y = x - int(x/y)*y" .
.PP
The Fortran logical and relational operators are
supported, with convenient modern C-like synonyms:
.BR ".and. " ( &
.RB "and " && ),
.BR ".or. " ( |
.RB "or " || ),
.BR ".not. " ( ! ),
.BR ".eq. " ( == ),
.BR ".ne. " ( != ),
.BR ".lt. " ( < ),
.BR ".le. " ( <= ),
.BR ".gt. " ( > ),
and
.BR ".ge. " ( >= ).
Letter case in the dotted operators is not significant.
Finally, the Fortran character string concatenation
operator,
.BR "//" ,
is handled.
.PP
One special name,
.BR defined ,
is recognized, in any letter case; it may be used either in
functional form,
.BR defined(name) ,
or in prefix operator form,
.BR "defined name" .
It evaluates to 1 if the name is defined (even if the value of
the name is zero), and otherwise, to 0.  Several
.B defined
operators can be used in a single expression; that is much more
convenient than a series of nested conditionals using
.B C#ifdef
and
.BR C#ifndef .
.PP
The
.B #(.\|.\|.)
form is only recognized in a comment line, and the next line is
converted to a comment (see the section
.I "MACRO EXPANSION"
below); the parentheses hold an expression involving Fortran
constants and preprocessor names.
.PP
Examples of expressions are
.PP
.RS
.nf
.B "C#if\0\0defined(_OS_UNIX)\0\0||\0\0defined\0_OS_VAXVMS\0\0||\
\0\0(WORDSIZE\0==\032)"
.PP
.B "C\0\0\0\0\0REAL\0A(#(MAXA**2)),\0B(#(MAXA\0%\032))"
.PP
.B "C\0\0\0\0\0INTEGER\0BITS(#(WORDSIZE))"
.fi
.RE
.\"======================================================================
.SH "MACRO EXPANSION"
.B fpp
supports a
.I reversible
argument-free macro expansion capability.  This involves pairs of
lines, the first a comment line containing the macro references
as strings of the form
.BR #(constant-expression) ,
and the second a non-comment Fortran statement.
.PP
The first line of the pair is always exactly preserved in the
output, while the second is replaced by the expansion of the
comment, with the first character
.IR deleted ,
to change the comment into a non-comment.  The original contents
of the second line are preserved as a comment with the prefix
.B C-fpp-
in a third output line.
.PP
This peculiar input line pairing is necessary to ensure
that the expansion is reversible.
.PP
The parentheses around the expression serve to distinguish
between macros and
.B fpp
preprocessor directives in comments, and serendipitously permit
the extension from simple names to arbitrary constant expressions
that can be evaluated by
.BR fpp .
.PP
Care must be taken in writing the input to ensure that any
expected expansion does not make the line longer than 72
characters;
.B fpp
has almost no knowledge of Fortran, and therefore cannot provide
correct line wrapping for it.  However, it will warn about long lines.
.PP
Similarly, macro expansion in a multi-line continued statement
should avoided, since it introduces comment lines between
continuation lines.  While such comments are legal in full
Fortran 77, they are illegal in subset Fortran 77, and in older
Fortrans, and may cause problems for other tools that process
Fortran code.
.PP
Here is a small example.  Given command-line definitions
.PP
.RS
.nf
.B FPTYPE='DOUBLE PRECISION'
.B MAXA=19
.B MAXB=25
.fi
.RE
.PP
then the input
.PP
.RS
.nf
.B "C\0\0\0\0\0#(FPTYPE)\0A(#(MAXA)),\0B(#(MAXB),#(MAXA**2))"
.B "\0\0\0\0\0\0REAL\0A(100),\0B(255,10000)"
.fi
.RE
.PP
is converted to the output
.PP
.RS
.nf
.B "C\0\0\0\0\0#(FPTYPE)\0A(#(MAXA)),\0B(#(MAXB),#(MAXA**2))"
.B "\0\0\0\0\0\0DOUBLE\0PRECISION\0A(19),\0B(25,361)"
.B "C-fpp-\0\0\0\0\0\0REAL\0A(100),\0B(255,10000)"
.fi
.RE
.PP
Fortran 77
.B PARAMETER
statements can be used to achieve similar effects, but in more
restricted circumstances.  In particular,
.B fpp
permits such expansions to happen in strings:
.PP
.RS
.nf
.B "C10000 FORMAT ('Host operating system = #(OS)')"
.B "10000 FORMAT ('Host operating system = UNIX')"
.fi
.RE
.PP
This may be awkward to achieve in standard Fortran.
.\"======================================================================
.SH "MESSAGE OUTPUT STATEMENTS"
Text can be written to
.I stderr
with either of
.PP
.RS
.nf
.BR "C#message " text
.PP
.BR "C#error " text
.fi
.RE
.PP
The difference between them is that
.B "#error"
sets an exit code of 1 (on POSIX and UNIX), and also sends the text to
.IR stdout .
This can be used to ensure that a preprocessing error forces a
compilation error if an attempt is later made to compile the
output source program.
.PP
The output of both directives is prefixed with the file name
and input line number to identify the origin of the message.
.PP
When
.B "#error"
is executed, processing is not terminated; instead,
.B fpp
tries to process the remaining input so as to uncover additional
errors in the same run.
.\"======================================================================
.SH "OUTPUT OF fpp"
The output contains an initial comment header of the form
.PP
.RS
.nf
\&\fBC-fpp-\0=================================================================
C-fpp-\0fpp\0version\01.0\0[10-Dec-1990]
C-fpp-\0Date:\0Sat\0Dec\0\08\023:06:30\0MST\01990
C-fpp-\0Directory:\0/u/sy/beebe
C-fpp-\0User:\0beebe@math.utah.edu
C-fpp-\0Macro:\0_OS_VAXVMS=1
C-fpp-\0Macro:\0FPTYPE=DOUBLE\0PRECISION
C-fpp-\0=================================================================\fP
.fi
.RE
.PP
These comments provide a record of the processing, including what
symbol definitions and macro values have been selected.
.PP
Input comments beginning
.PP
.RS
.nf
.B "C-fpp-"
.fi
.RE
.PP
are flushed.  Thus, any existing comment header is always
replaced by a new header.
.PP
Each command-line
.B name=value
or
.B \-Dname
setting, and each input definition directive
.PP
.RS
.nf
.B "C#define name value"
.fi
.RE
.PP
produce an output comment of the form
.PP
.RS
.nf
.B "C-fpp- Macro: name=value"
.fi
.RE
.PP
Thus, all output lines beginning
.PP
.RS
.nf
.B "C-fpp- Macro:"
.fi
.RE
.PP
document which names have been defined.
.PP
A
.B C#undefine
statement results in output like
.PP
.RS
.nf
.B "C-fpp- Macro: name=--UNDEFINED--"
.fi
.RE
.\"======================================================================
.SH "PREDEFINED SYMBOLS"
Each implementation of
.B fpp
predefines a few symbols that can be tested and used in
conditionals to select machine-specific code sections.  The
predefined symbols are provided to
.B fpp
ahead of any user-defined ones.  Since later redefinitions
override earlier ones, predefined symbols can always be changed
by the user.
.PP
Following ISO Standard C/C++, predefined symbols always begin with two
underscores, or an underscore and an uppercase letter; such names
are reserved for the local implementation.
Predefined symbols that do not follow this convention are
.IR forbidden .
This requirement makes it possible to distinguish separate name
spaces for the user and for the implementation, preventing
surprises from unexpected substitutions that happen when code is
moved to a new environment.
.PP
The complete set of definitions is always recorded in the output
header; they can easily be displayed on
.I stdout
by giving
.B fpp
an empty input file:
.PP
.RS
.nf
.BI fpp /dev/null
.fi
.RE
.PP
.B fpp
always predefines
.I "exactly one"
major operating-system symbol:
.PP
.RS
.nf
.B _OS_PCDOS
.B _OS_TOPS20
.B _OS_UNIX
.B _OS_VAXVMS
.fi
.RE
.PP
For
.BR _OS_UNIX ,
exactly one minor operating-system variant may also be defined:
.PP
.RS
.nf
.B _AIX
.B _AIX370
.B _BSD
.B _DARWIN
.B _FREEBSD
.B _GOULD
.B _HPUX
.B _IRIX
.B _LINUX
.B _MACH
.B _MIPS
.B _NETBSD
.B _OPENBSD
.B _OSF1
.B _RHAPSODY
.B _STARDENT
.B _SUNOS
.B _ULTRIX
.fi
.RE
.PP
Additional architectural variants may be defined on some systems:
.RS
.nf
.B _GNU_LINUX
.B _HPPA
.B _IBM_3090
.B _IBM_PS_2
.B _IBM_RS_6000
.B _IBM_RT
.B _IRIX64
.B _M68K
.B _MACOSX
.B _NEXT
.B _POSIX
.B _PPC
.B _STARDENT_1500
.B _STARDENT_3000
.B _SUN3
.B _SUN386
.B _SUN4
.B _VAX
.B _X86
.fi
.RE
The operating system name as returned by
.BR uname (1)
is recorded as the value of
.BR _OS_NAME ,
after collapsing characters other than letters, digits, period, and
hyphen to underscores.
.RS
.nf
.fi
.RE
.PP
The operating system level, usually a string of digits separated by
periods and/or hyphens, is recorded as the value of
.BR _OS_LEVEL .
On most systems, it too is obtained from
.BR uname (1).
.PP
The host CPU architecture is recorded as the value of
.BR _ARCH ,
one of
.RS
.nf
.B Alpha
.B Convex
.B Cray
.B Gould
.B IA-64
.B IBM-3090
.B MIPS
.B Motorola-68K
.B PA-RISC
.B PowerPC
.B SPARC
.B Stardent
.B VAX
.B unknown
.B x86
.fi
.RE
.PP
Host byte addressing order is defined by one of
these:
.PP
.RS
.nf
.B _BIG_ENDIAN
.B _LITTLE_ENDIAN
.fi
.RE
.PP
Big-endian addressing is used by IBM, Motorola, and most RISC systems.
Little-endian addressing is used by the Intel x86, HP/Intel IA-64, DEC
VAX, and Compaq/DEC Alpha architectures.  Although a few RISC
architectures support both endian orders, a fixed choice is always
made by the operating system to ensure consistent byte ordering in
binary files and network traffic.
.PP
Host floating-point architecture must be defined by
.RS
.nf
.B _IEEE_754
.fi
.RE
on those machines that have IEEE 754 floating-point arithmetic.
.PP
If the Fortran implementation supports NAMELIST I/O, the symbol
.RS
.nf
.B _NAMELIST
.fi
.RE
must be defined.
.PP
To ensure standardization, all such names must be registered
with the author of
.BR fpp ,
and will be listed in these manual pages.
.PP
Following ISO Standard C/C++, four standard permanent symbols are always
defined; these each have two leading and two trailing underscores.
Permanent symbols always begin with two underscores, and once defined,
may not be undefined, or redefined, by the user.
.RS
.\"---------------------------------------------------------------------
.TP \w'\fB_\|_TIMEZONE_\|_\fP'u+2n
.B _\|_DATE_\|_
Current calendar date in the form
.IR "Mmm dd yyyy" .
The month field is alphabetic, and the day number field has
a leading blank if the day is less than 10.
.\"---------------------------------------------------------------------
.TP
.B _\|_FILE_\|_
Current input file filename.
.\"---------------------------------------------------------------------
.TP
.B _\|_LINE_\|_
Current input file line number.
.\"---------------------------------------------------------------------
.TP
.B _\|_TIME_\|_
Wall-clock time in the form
.IR hh:mm:ss .
.RE
.PP
In addition to those four,
.B fpp
sets two related values:
.RS
.\"---------------------------------------------------------------------
.TP \w'\fB_\|_TIMEZONE_\|_\fP'u+2n
.B _\|_ISO_DATE_\|_
Ten-character ISO 8601 date in the form
.IR YYYY-MM-DD .
.\"---------------------------------------------------------------------
.TP
.B _\|_TIMEZONE_\|_
Three-letter time zone abbreviation, such as MDT for
Mountain Daylight Time, or GMT for Greenwich Mean Time.
.\"---------------------------------------------------------------------
.RE
.PP
The symbols
.BR _\|_DATE_\|_ ,
.BR _\|_ISO_DATE_\|_ ,
.BR _\|_TIME_\|_ ,
and
.B _\|_TIMEZONE_\|_
are set only once, at the start of execution of
.BR fpp .
These values can be conveniently used to generate output
stamped with the time of compilation:
.PP
.RS
.nf
\&\fBC\0\0\0\0\0WRITE\0(*,*)\0'Processed\0on\0#(__DATE__)\0at\0#(__TIME__)\0\
#(__TIMEZONE__)'
\0\0\0\0\0\0WRITE\0(*,*)\0'Processed\0on\0???\0??\0????\0at\0??:??:??\0???'\fP
.fi
.RE
.PP
might produce
.PP
.RS
.nf
\&\fBC\0\0\0\0\0WRITE\0(*,*)\0'Processed\0on\0#(__DATE__)\0at\0#(__TIME__)\0\
#(__TIMEZONE__)'
\0\0\0\0\0\0WRITE\0(*,*)\0'Processed\0on\0Dec\010\01990\0at\009:10:07\0MST'
C-fpp-\0\0\0\0\0\0WRITE\0(*,*)\0'Processed\0on\0???\0??\0????\0at\0??:??:??\
\0???'\fP
.fi
.RE
.PP
.\"======================================================================
.SH "DIAGNOSTICS"
Diagnostics are issued to
.I stderr
if unclosed conditionals, out-of-place conditional branches,
errors in preprocessor expressions, or long lines, are detected.
Attempts to redefine permanent macros (any that begin with two
underscores) produce an error message.  Debug output requested by a
command-line option will be sent to
.IR stderr .
.PP
Directives
.RS
.nf
.BR C#error " text"
.fi
.RE
that are executed send their text argument to
.I stderr
and to
.IR stdout ,
and cause a later exit code of 1 (on POSIX and UNIX).
.PP
Directives
.RS
.nf
.BR C#message " text"
.fi
.RE
that are executed send their text argument to
.IR stderr .
.PP
Diagnostic messages have the format
.RS
.I "filename:linenumber:message"
.RE
commonly used by UNIX, POSIX, and GNU software.
Advanced text editors, such as
.BR emacs (1),
recognize that format, and allow the user to move to the error
location with a couple of keystrokes.
.\"======================================================================
.SH "SEE ALSO"
.BR awk (1),
.BR c++ (1),
.BR CC (1),
.BR cc (1),
.BR cpp (1),
.BR cxx (1),
.BR d2s (1),
.BR diff (1),
.BR dtoq (1),
.BR dtos (1),
.BR emacs (1),
.BR f77 (1),
.BR f90 (1),
.BR f95 (1),
.BR fsplit (1),
.BR ftnchek (1),
.BR g++ (1),
.BR g77 (1),
.BR gawk (1),
.BR gcc (1),
.BR lf95 (1),
.BR m4 (1),
.BR mawk (1),
.BR nagf90 (1),
.BR nagf95 (1),
.BR nawk (1),
.BR pfort (1),
.BR pgf77 (1),
.BR pgf90 (1),
.BR pretty (1),
.BR qtod (1),
.BR s2d (1),
.BR sf3lex (1),
.BR sf3pretty (1),
.BR stod (1),
.BR strsf3 (1),
.BR struct (1),
.BR tidy (1),
.BR toolpack (1),
.BR uname (1),
.BR xlf (1),
.BR xlf90 (1),
.BR xlf95 (1),
.BR xsf3 (1).
.PP
American National Standards Institute,
.I "American National Standard programming language FORTRAN:"
.IR "approved April 3, 1978",
ANSI X3.9-1978, New York, 1978.
Revision of ANSI X3.9-1966.
.PP
S. P. Harbison and G. L. Steele, Jr.,
.IR "C: A Reference Manual" ,
4th ed., Prentice-Hall, 1995.
.PP
B. W. Kernighan and D. M. Ritchie,
.IR "The C Programming Language" ,
2nd ed., Prentice-Hall, 1988.
.PP
American National Standards Inst.,
.IR "American National Standard for Information Systems \(em Programming Language \(em C" ,
ANSI X3.159-1989, New York, 1990.
.PP
International Organization for Standardization,
.IR "ISO/IEC 9899:1990: Programming languages \(em C" ,
Geneva, Switzerland, 1990.
.PP
International Organization for Standardization,
.IR "International standard: information, technology, programming languages, Fortran" ,
ISO/IEC 1539:1991,
Geneva, 1991.
.PP
IEEE,
.I "9945-2: 1993 (ISO/IEC) [IEEE/ANSI Std 1003.2-1992 and IEEE/ANSI"
.I "1003.2a-1992] Information Technology \(em Portable Operating System"
.IR "Interface (POSIX(\(rg)) \(em Part 2: Shell and Utilities",
New York, 1993.
.PP
American National Standards Institute,
.I "ANSI/ISO/IEC 1539-1:1997: Information technology \(em"
.IR "Programming languages \(em Fortran \(em Part 1: Base language"
.PP
Jeanne C. Adams, Walter S. Brainerd, Jeanne T. Martin, Brian T. Smith,
and Jerrold L. Wagener,
.IR "Fortran 95 Handbook: Complete ISO/ANSI Reference" ,
MIT Press, Cambridge, MA,
1997.
ISBN 0-262-51096-0
.PP
International Organization for Standardization,
.IR "ISO/IEC 14882:1998: Programming languages \(em C++"
Geneva, Switzerland, 1998.
.PP
International Organization for Standardization,
.IR "ISO/IEC 9899:1999: Programming languages \(em C" ,
Geneva, Switzerland, 1999.
.\"======================================================================
.SH "AUTHOR"
.nf
Nelson H. F. Beebe
Center for Scientific Computing
University of Utah
Department of Mathematics, 322 INSCC
155 S 1400 E RM 233
Salt Lake City, UT 84112-0090
USA
Email: \fCbeebe@math.utah.edu\fP, \fCbeebe@acm.org\fP,
.if n .ti +\w'Email:\ 'u
.if t .ti +\w'Email:\ 'u
\fCbeebe@computer.org\fP, \fCbeebe@ieee.org\fP (Internet)
WWW URL: \fChttp://www.math.utah.edu/~beebe\fP
Telephone: +1 801 581 5254
FAX: +1 801 585 1640, +1 801 581 4148
.fi
.\"=====================================================================
.SH AVAILABILITY
.B fpp
is freely available; its master distribution can
be found at
.PP
.RS
.nf
\&\fCftp://ftp.math.utah.edu/pub/misc/\fP
\&\fChttp://www.math.utah.edu/pub/misc/\fP
.fi
.RE
.PP
in the file
.I fpp-x.yy.tar.gz
where
.I x.yy
is the current version.  Other distribution
formats are usually available at the same
location.
.PP
That site is mirrored to several other Internet
archives, so you may also be able to find it
elsewhere on the Internet; try searching for the
string
.I fpp
at one or more of the popular Web search sites,
such as
.PP
.RS
.nf
\&\fChttp://altavista.digital.com/
http://search.microsoft.com/us/default.asp
http://www.dejanews.com/
http://www.dogpile.com/index.html
http://www.euroseek.net/page?ifl=uk
http://www.excite.com/
http://www.go2net.com/search.html
http://www.google.com/
http://www.hotbot.com/
http://www.infoseek.com/
http://www.inktomi.com/
http://www.lycos.com/
http://www.northernlight.com/
http://www.snap.com/
http://www.stpt.com/
http://www.yahoo.com/\fP
.fi
.RE
.\"=====================================================================
.SH COPYRIGHT
.nf
\fC########################################################################
########################################################################
########################################################################
###                                                                  ###
### fpp: cpp-like reversible preprocessor filter for Fortran and     ###
### SFTRAN3 code                                                     ###
###                                                                  ###
###        Copyright (C) 1990, 1993, 2001 Nelson H. F. Beebe         ###
###                                                                  ###
### This program is covered by the GNU General Public License (GPL), ###
### version 2 or later, available as the file COPYING in the program ###
### source distribution, and on the Internet at                      ###
###                                                                  ###
###               ftp://ftp.gnu.org/gnu/GPL                          ###
###                                                                  ###
###               http://www.gnu.org/copyleft/gpl.html               ###
###                                                                  ###
### This program is free software; you can redistribute it and/or    ###
### modify it under the terms of the GNU General Public License as   ###
### published by the Free Software Foundation; either version 2 of   ###
### the License, or (at your option) any later version.              ###
###                                                                  ###
### This program is distributed in the hope that it will be useful,  ###
### but WITHOUT ANY WARRANTY; without even the implied warranty of   ###
### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the    ###
### GNU General Public License for more details.                     ###
###                                                                  ###
### You should have received a copy of the GNU General Public        ###
### License along with this program; if not, write to the Free       ###
### Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   ###
### MA 02111-1307 USA.                                               ###
########################################################################
########################################################################
########################################################################\fP
.fi
.\"==============================[The End]==============================
